@@ -13,12 +13,13 @@ import datetime
 import inspect
 import os
 
-import matplotlib as mpl
+# backend for display in GitHub Actions
 if os.environ.get('DISPLAY','') == '':
+    import matplotlib as mpl
     print('no display found. Using non-interactive Agg backend')
     mpl.use('Agg')
 
-# used to drwa lines on the figure
+# used to draw lines on the figure
 import matplotlib.lines as lines
 # matplotlib import used in setting figure and axes
 import matplotlib.pyplot as plt
@@ -34,7 +35,6 @@ import configobj
 # import the validator
 from validate import Validator
 
-
 # set up the base string depending if we are running Python 2 or Python 3
 try:
     # check whether python knows about 'basestring'
@@ -98,6 +98,7 @@ class FigPager:
         overwrite=False,
         sharex=False,
         sharey=False,
+        draft=True,
     ):
 
         """
@@ -126,7 +127,8 @@ class FigPager:
             direction: (string) (optional) subplot creation direction. Default is left-to-right.
             overwrite: (boolean) (optional) Boolean on whether to overwrite existing output. Default is True
             sharex: share x axes in subplots. Default is False
-            sharey:share y axes in subplots.Default is False
+            sharey:share y axes in subplots. Default is False
+            draft: Add draft stamp if available from ini. Default is True
         """
 
         # obtain the caller path
@@ -226,6 +228,7 @@ class FigPager:
         self.width_ratios = width_ratios
         self.sharex = sharex
         self.sharey = sharey
+        self.draft = draft
 
         # figure attributes
         self.fig = None
@@ -239,6 +242,8 @@ class FigPager:
 
         # save current filename as possible next filename base if not multipage
         self.new_fname = self.outfile
+        # list to append multiple filenames
+        self.newfnameall = []
 
         # saves current figure count for use in non multipage
         self.fignumber = 1
@@ -420,6 +425,7 @@ class FigPager:
                 "rotation": "integer",
                 "line_position_start": "float_list_value",
                 "line_position_end": "float_list_value",
+                "line_position_offset": "float_list_value",
                 "line_min": "float",
                 "line_max": "float",
                 "line_length": "string",
@@ -428,6 +434,7 @@ class FigPager:
                 "linestyle": "string",
                 "image_path": "string",
                 "image_position": "float_list_value",
+                "image_position_offset": "float_list_value",
             }
 
             data = myfile.read()
@@ -590,6 +597,15 @@ class FigPager:
                 self.frameheight = (
                     self.pageheight_inch - self.topmargin - self.bottommargin
                 )
+            else:
+                self.leftmargin = 0
+                self.rightmargin = 0
+                self.topmargin = 0
+                self.bottommargin = 0
+                self.marginpad = 0
+                self.framewidth = self.pagewidth_inch
+                self.frameheight = self.pageheight_inch
+
 
             self.source_path = self._parse_option("Layout", "Margin", "source_path")
             self.source_path_position = [
@@ -769,6 +785,23 @@ class FigPager:
         """
         pos = self._parse_option("Lines", label, "line_position_start")
         pos2 = self._parse_option("Lines", label, "line_position_end")
+
+        if self._parse_option("Lines", label, "image_position_offset") is not None:
+            pos[0] = pos[0] + float(
+                self._parse_option("Images", label, "image_position_offset")[0]
+            )
+            pos[1] = pos[1] + float(
+                self._parse_option("Images", label, "image_position_offset")[1]
+            )
+
+            pos2[0] = pos2[0] + float(
+                self._parse_option("Images", label, "image_position_offset")[0]
+            )
+            pos2[1] = pos2[1] + float(
+                self._parse_option("Images", label, "image_position_offset")[1]
+            )
+
+
         l1 = lines.Line2D(
             [pos[0] / self.pagewidth_inch, pos2[0] / self.pagewidth_inch],
             [pos[1] / self.pageheight_inch, pos2[1] / self.pageheight_inch],
@@ -794,6 +827,15 @@ class FigPager:
         """
         pos = self._parse_option("Images", label, "image_position")
         fname = self._parse_option("Images", label, "image_path")
+
+        if self._parse_option("Images", label, "image_position_offset") is not None:
+            pos[0] = pos[0] + float(
+                self._parse_option("Images", label, "image_position_offset")[0]
+            )
+            pos[1] = pos[1] + float(
+                self._parse_option("Images", label, "image_position_offset")[1]
+            )
+
         if fname:
             filename, file_extension = os.path.splitext(fname)
             img = plt.imread(fname, format=file_extension)
@@ -803,6 +845,7 @@ class FigPager:
             width, height = self.fig.get_size_inches() * self.fig.get_dpi()
             # We're specifying the position and size in figure coordinates, so the image
             # will shrink/grow as the figure is resized.
+            # zorder = 1 places the images on top
             ax = self.fig.add_axes(
                 [
                     pos[0] / self.pagewidth_inch,
@@ -810,7 +853,7 @@ class FigPager:
                     width1 / width,
                     height1 / height,
                 ],
-                anchor="SW",
+                anchor="SW",zorder=1,
             )
             ax.imshow(img)
             ax.axis("off")
@@ -970,6 +1013,8 @@ class FigPager:
         # add any layout set text here
         for k in self.config["Text"].keys():
             if self._parse_option("Text", k, "text") is not None:
+                if 'draft' in self._parse_option("Text", k, "text").lower():
+                    if not self.draft: continue
                 self._text_from_label("Text", k, self._parse_option("Text", k, "text"))
 
         # add any layout set images here
@@ -983,6 +1028,9 @@ class FigPager:
         # add any layout set watermarks here
         for k in self.config["Watermark"].keys():
             if self._parse_option("Watermark", k, "text") is not None:
+                # check for draft watermark status and whether user has overridden it
+                if 'draft' in self._parse_option("Watermark", k, "text").lower():
+                    if not self.draft: continue
                 self._text_from_label(
                     "Watermark", k, self._parse_option("Watermark", k, "text")
                 )
@@ -1128,6 +1176,48 @@ class FigPager:
         Returns: fig; figure instance, ax; axes instances, gs; GridSpec, self.transform; Figure Transform
 
         """
+
+        if self.type in ["pdf", "pgf"]:
+            try:
+                self.pdf.savefig(
+                    self.fig,
+                    dpi=self.dpi,
+                    facecolor=self.facecolor,
+                    edgecolor=self.edgecolor,
+                    orientation=self.orientation,
+                    transparent=self.transparent,
+                    bbox_inches=self.bbox_inches,
+                    pad_inches=self.pad_inches,
+                    metadata=self.metadata,
+                )
+            except AttributeError as a:
+                if str(a) == "'NoneType' object has no attribute 'endStream'":
+                    raise AttributeError("Cannot add a new page to a closed pdf file.")
+        elif self.type is not None:
+            # number 02, 03 etc '{:02}'.format(1)
+            plt.savefig(
+                self.new_fname,
+                dpi=self.dpi,
+                facecolor=self.facecolor,
+                edgecolor=self.edgecolor,
+                orientation=self.orientation,
+                transparent=self.transparent,
+                bbox_inches=self.bbox_inches,
+                pad_inches=self.pad_inches,
+                metadata=self.metadata,
+            )
+            # append multiple filenames
+            self.newfnameall.append(self.new_fname)
+
+            filename, file_extension = os.path.splitext(self.outfile)
+            self.fignumber = self.fignumber + 1
+            self.new_fname = "{}_{}{}".format(
+                filename, "{:02}".format(self.fignumber), file_extension
+            )
+        else:
+            plt.show()
+        plt.close(self.fig)
+
         if paper_size is not None:
             self.paper_size = paper_size
 
@@ -1190,44 +1280,6 @@ class FigPager:
             # update from layout
             self._update_from_layout()
 
-        if self.type in ["pdf", "pgf"]:
-            try:
-                self.pdf.savefig(
-                    self.fig,
-                    dpi=self.dpi,
-                    facecolor=self.facecolor,
-                    edgecolor=self.edgecolor,
-                    orientation=self.orientation,
-                    transparent=self.transparent,
-                    bbox_inches=self.bbox_inches,
-                    pad_inches=self.pad_inches,
-                    metadata=self.metadata,
-                )
-            except AttributeError as a:
-                if str(a) == "'NoneType' object has no attribute 'endStream'":
-                    raise AttributeError("Cannot add a new page to a closed pdf file.")
-        elif self.type is not None:
-            # probably number 02, 03 etc '{:02}'.format(1)
-            plt.savefig(
-                self.new_fname,
-                dpi=self.dpi,
-                facecolor=self.facecolor,
-                edgecolor=self.edgecolor,
-                orientation=self.orientation,
-                transparent=self.transparent,
-                bbox_inches=self.bbox_inches,
-                pad_inches=self.pad_inches,
-                metadata=self.metadata,
-            )
-
-            filename, file_extension = os.path.splitext(self.outfile)
-            self.fignumber = self.fignumber + 1
-            self.new_fname = "{}_{}{}".format(
-                filename, "{:02}".format(self.fignumber), file_extension
-            )
-        else:
-            plt.show()
-        plt.close(self.fig)
         self.fig, self.ax, self.gs, self.transform = self.draw_page()
 
         return self.fig, self.ax, self.gs, self.transform
@@ -1253,6 +1305,11 @@ class FigPager:
                     pad_inches=self.pad_inches,
                     metadata=self.metadata,
                 )
+                self.newfnameall.append(self.new_fname)
+
+                # update the outfile name if needed
+                if self.newfnameall:
+                    self.outfile = '\n'.join(self.newfnameall)
             else:
                 self.pdf.savefig(
                     self.fig,
@@ -1277,6 +1334,8 @@ class FigPager:
 
                 # Remember to close the object - otherwise the file will not be usable
                 self.pdf.close()
+
+
         else:
             plt.show()
             plt.close(self.fig)
